const isChrome = !!window.chrome && (!!window.chrome.webstore || !!window.chrome.runtime)
const extName = isChrome ? 'chrome' : 'firefox'
const rootUrl = 'https://x.tayl.app'

const activeTabQuery = {
  active: true,
  currentWindow: true,
}

const badgeBackgroundColor = {color: [25, 145, 255, 255]}
const isFirefox = typeof InstallTrigger !== 'undefined'

const setTabActiveState = (tabId, isActive) => {
  if (isFirefox) return
  const text = isActive ? 'ON' : ''
  browser.browserAction.setBadgeBackgroundColor(badgeBackgroundColor)
  browser.browserAction.setBadgeText({tabId, text})
}

const handleToggle = async ({id}) => {
  const isActive = await browser.tabs.sendMessage(id, {
    ext: 'tayl',
    type: 'toggleExt',
  })
  setTabActiveState(id, isActive)
}

const handleOpen = async () => {
  const [tab] = await browser.tabs.query(activeTabQuery)
  handleToggle(tab)
}

const unmount = async () => {
  const [tab] = await browser.tabs.query(activeTabQuery)
  handleToggle(tab)
}

const getUserSettings = async () => {
  return await browser.storage.sync.get()
}

const getCurrentTabInfo = async () => {
  const [tab] = await browser.tabs.query(activeTabQuery)
  return {
    ...tab,
  }
}

const setUserSettings = async ({ userSettings }) => {
  await browser.storage.sync.set(userSettings)
  return true
}

const getApiTokenInfo = async ({ apiToken }) => {
  return await postJson(`${rootUrl}/api-token/info`, {apiToken})
}

const getReadable = async ({ url, markup }) => {
  return await postJson(`${rootUrl}/readable`, {
    data: {
      url,
      markup,
    },
    headers: {
      'x-source': `extension:${extName}`,
    },
  })
}

const sendToTayl = async ({apiToken, url, markup}) => {
  return await postJson(`${rootUrl}/submit`, {
    apiToken,
    data: {
      url,
      markup,
    },
    headers: {
      'x-source': `extension:${extName}`,
    },
  })
}

const setApiKey = async ({ apiKey }) => {
  await browser.storage.sync.set({ apiKey })
  return true
}

const handleStorageChanged = async (changes, area) => {
  const userSettings = await getUserSettings()
  sendMessageToAllTabs({
    ext: 'tayl',
    type: 'storageChanged',
    userSettings,
  })
}

const sendMessageToAllTabs = async message => {
  const tabs = await browser.tabs.query({})
  const promises = tabs.map(({ id }) => chrome.tabs.sendMessage(id, message))
  return await Promise.all(promises)
}

async function postJson(url, options) {
  return await jsonFetch(url, options)
}

async function jsonFetch(url, options = {}, method = 'post') {
  const {data, idToken, apiToken, ...props} = options

  const req = {
    method,
    ...props,
  }

  req.headers = {
    ...req.headers,
    Accept: 'application/json',
    'Content-Type': 'application/json',
  }

  if (idToken) {
    req.headers['x-id-token'] = idToken
  }

  if (apiToken) {
    req.headers['x-api-token'] = apiToken
  }

  if (data) req.body = JSON.stringify(data)

  const res = await fetch(url, req)
  const text = await res.text()
  const json = text ? JSON.parse(text) : {}

  if (res.status === 200) return json
  throw json
}

browser.contextMenus.create({
  id: 'TAYL',
  title: 'Open in TAYL',
  contexts: ['all'],
})

browser.contextMenus.onClicked.addListener(handleOpen)
browser.browserAction.onClicked.addListener(handleToggle)

const msgHandlers = {
  unmount,
  getUserSettings,
  getCurrentTabInfo,
  setUserSettings,
  getApiTokenInfo,
  getReadable,
  sendToTayl,
}

const externalMsgHandlers = {
  setApiKey,
}

browser.runtime.onMessage.addListener(async msg => {
  if (msg.ext !== 'tayl') return false
  const handler = msgHandlers[msg.type]
  if (!handler) return
  return await handler(msg)
})

browser.runtime.onMessageExternal.addListener(async msg => {
  if (msg.ext !== 'tayl') return false
  const handler = externalMsgHandlers[msg.type]
  if (!handler) return
  return await handler(msg)
})

browser.storage.onChanged.addListener(handleStorageChanged)
